@@ -17,6 +17,17 @@
 
 /* Written by David MacKenzie and Torbjorn Granlund. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -123,7 +134,7 @@ eaccess_stat (statp, mode)
       ngroups = getgroups ((int) NGROUPS_MAX, groups);
 #endif
     }
-  
+
   /* The super-user can read and write any file, and execute any file
      that anyone can execute. */
   if (euid == 0 && ((mode & X_OK) == 0 || (statp->st_mode & 0111)))
