@@ -15,7 +15,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* 
+/*
               |     		      user
               | unchanged                 explicit
  -------------|-------------------------+-------------------------|
@@ -28,6 +28,17 @@
 
    Written by David MacKenzie <djm@gnu.ai.mit.edu>. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/types.h>
@@ -83,10 +94,10 @@ static char *username;
 static char *groupname;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -95,8 +106,8 @@ static struct option const long_options[] =
   {"silent", no_argument, 0, 'f'},
   {"quiet", no_argument, 0, 'f'},
   {"verbose", no_argument, 0, 'v'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -135,21 +146,21 @@ main (argc, argv)
 	  verbose = 1;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind >= argc - 1)
-    usage ();
+    usage (1);
 
   e = parse_user_spec (argv[optind], &user, &group, &username, &groupname);
   if (e)
@@ -281,11 +292,30 @@ describe_change (file, changed)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-Rcfv] [--recursive] [--changes] [--silent] [--quiet]\n\
-       [--verbose] [--help] [--version] [user][:.][group] file...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... OWNER[.[GROUP]] FILE...\n\
+  or:  %s [OPTION]... .[GROUP] FILE...\n\
+",
+	      program_name, program_name);
+      printf ("\
+\n\
+  -c, --changes           be verbose whenever change occurs\n\
+  -f, --silent, --quiet   suppress most error messages\n\
+  -v, --verbose           explain what is being done\n\
+  -R, --recursive         change files and directories recursively\n\
+      --help              display this help and exit\n\
+      --version           output version information and exit\n\
+\n\
+Owner is unchanged if missing.  Group is unchanged if missing, but changed\n\
+to login group if implied by a period.  A colon may replace the period.\n");
+    }
+  exit (status);
 }
