@@ -20,7 +20,7 @@
 /* Options:
 
    Numbers can be followed by a multiplier:
-   b=512, k=1024, w=2, xm=number m
+   b=512, c=1, k=1024, w=2, xm=number m
 
    if=FILE			Read from FILE instead of stdin.
    of=FILE			Write to FILE instead of stdout; don't
@@ -53,6 +53,17 @@
    sync				Pad every input block to size of ibs with
 				trailing NULs. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <ctype.h>
 
@@ -68,6 +79,7 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <getopt.h>
+
 #include "system.h"
 #include "version.h"
 
@@ -309,15 +321,15 @@ static unsigned char const ebcdic_to_ascii[] =
 };
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -340,14 +352,14 @@ main (argc, argv)
   /* Decode arguments. */
   scanargs (argc, argv);
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   apply_translations ();
 
@@ -382,7 +394,7 @@ main (argc, argv)
     }
   else
     output_file = "standard output";
-  
+
 #ifdef _POSIX_VERSION
   sigaction (SIGINT, NULL, &sigact);
   if (sigact.sa_handler != SIG_IGN)
@@ -693,7 +705,7 @@ copy_simple (buf, nread)
 	nfree = nread;
 
       bcopy (start, obuf + oc, nfree);
-	    
+
       nread -= nfree;		/* Update the number of bytes left to copy. */
       start += nfree;
       oc += nfree;
@@ -810,7 +822,7 @@ scanargs (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
@@ -823,7 +835,7 @@ scanargs (argc, argv)
       if (val == NULL)
 	{
 	  error (0, 0, "unrecognized option `%s'", name);
-	  usage ();
+	  usage (1);
 	}
       *val++ = '\0';
 
@@ -862,7 +874,7 @@ scanargs (argc, argv)
 	  else
 	    {
 	      error (0, 0, "unrecognized option `%s=%s'", name, val);
-	      usage ();
+	      usage (1);
 	    }
 	}
     }
@@ -905,6 +917,8 @@ loop:
     case 'b':
       n *= 512;
       goto loop;
+    case 'c':
+      goto loop;
     case 'k':
       n *= 1024;
       goto loop;
@@ -946,7 +960,7 @@ parse_conversion (str)
       if (conversions[i].convname == NULL)
 	{
 	  error (0, 0, "%s: invalid conversion", str);
-	  usage ();
+	  usage (1);
 	}
       str = new;
   } while (new != NULL);
@@ -1032,7 +1046,7 @@ print_stats ()
   fprintf (stderr, "%u+%u records in\n", r_full, r_partial);
   fprintf (stderr, "%u+%u records out\n", w_full, w_partial);
   if (r_truncate > 0)
-    fprintf (stderr, "%u truncated block%s\n", r_truncate,
+    fprintf (stderr, "%u truncated record%s\n", r_truncate,
 	     r_truncate == 1 ? "" : "s");
 }
 
@@ -1056,15 +1070,43 @@ interrupt_handler ()
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [if=file] [of=file] [ibs=bytes] [obs=bytes] [bs=bytes] [cbs=bytes]\n\
-       [skip=blocks] [seek=blocks] [count=blocks]\n\
-       [conv={ascii,ebcdic,ibm,block,unblock,lcase,ucase,swab,noerror,notrunc,\n\
-       sync}] [--help] [--version]\n\
-Numbers can be followed by a multiplier:\n\
-b=512, k=1024, w=2, xm=number m\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]...\n", program_name);
+      printf ("\
+\n\
+  bs=BYTES        force ibs=BYTES and obs=BYTES\n\
+  cbs=BYTES       convert BYTES bytes at a time\n\
+  conv=KEYWORDS   convert the file as per the comma separated keyword list\n\
+  count=BLOCKS    copy only BLOCKS input blocks\n\
+  ibs=BYTES       read BYTES bytes at a time\n\
+  if=FILE         read from FILE instead of stdin\n\
+  obs=BYTES       write BYTES bytes at a time\n\
+  of=FILE         write to FILE instead of stdout, don't truncate file\n\
+  seek=BLOCKS     skip BLOCKS obs-sized blocks at start of output\n\
+  skip=BLOCKS     skip BLOCKS ibs-sized blocks at start of input\n\
+      --help      display this help and exit\n\
+      --version   output version information and exit\n\
+\n\
+BYTES may be suffixed: by xM for multiplication by M, by c for x1,\n\
+by w for x2, by b for x512, by k for x1024.  Each KEYWORD may be:\n\
+\n\
+  ascii     from EBCDIC to ASCII\n\
+  ebcdic    from ASCII to EBCDIC\n\
+  ibm       from ASCII to alternated EBCDIC\n\
+  block     pad newline-terminated records with spaces to cbs-size \n\
+  unblock   replace trailing spaces in cbs-size records with newline\n\
+  lcase     change uppercase to lowercase\n\
+  ucase     change lowercase to uppercase\n\
+  swab      swap every pair of input bytes\n\
+  noerror   continue after read errors\n\
+  sync      pad every input block with NULs to ibs-size\n");
+    }
+  exit (status);
 }
