@@ -32,6 +32,17 @@
 
    Written by David MacKenzie <djm@gnu.ai.mit.edu> */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <getopt.h>
@@ -40,7 +51,6 @@
 #include "system.h"
 #include "version.h"
 
-char *strstr ();
 char *xmalloc ();
 char *xstrdup ();
 void error ();
@@ -110,10 +120,10 @@ static struct fs_type_list *fs_exclude_list;
 static struct mount_entry *mount_list;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -123,8 +133,8 @@ static struct option const long_options[] =
   {"portability", no_argument, &posix_format, 1},
   {"type", required_argument, 0, 't'},
   {"exclude-type", required_argument, 0, 'x'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -174,18 +184,18 @@ main (argc, argv)
 	  add_excluded_fs_type (optarg);
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind != argc)
     {
@@ -464,12 +474,28 @@ excluded_fstype (fstype)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-aikPv] [-t fstype] [-x fstype] [--all] [--inodes]\n\
-\t[--type=fstype] [--exclude-type=fstype] [--kilobytes] [--portability]\n\
-\t[--help] [--version] [path...]\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION] [PATH]...\n", program_name);
+      printf ("\
+\n\
+  -a, --all                 include filesystems having 0 blocks\n\
+  -i, --inodes              list inode information instead of block usage\n\
+  -k, --kilobytes           use 1024 blocks, not 512 despite POSIXLY_CORRECT\n\
+  -t, --type TYPE           limit the listing to TYPE filesystems type\n\
+  -x, --exclude-type TYPE   limit the listing to not TYPE filesystems type\n\
+  -v                        (ignored)\n\
+  -P, --portability         use the POSIX output format\n\
+      --help                display this help and exit\n\
+      --version             output version information and exit\n\
+\n\
+If no PATHs are given, list all currently mounted filesystems.\n");
+    }
+  exit (status);
 }
