@@ -39,6 +39,18 @@
 #ifdef _AIX
  #pragma alloca
 #endif
+
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -156,10 +168,10 @@ static int (*xstat) ();
 static int exit_status;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -173,25 +185,42 @@ static struct option const long_options[] =
   {"separate-dirs", no_argument, &opt_separate_dirs, 1},
   {"summarize", no_argument, &opt_summarize_only, 1},
   {"total", no_argument, &opt_combined_arguments, 1},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
 static void
-usage (reason)
+usage (status, reason)
+     int status;
      char *reason;
 {
   if (reason != NULL)
-    fprintf (stderr, "%s: %s\n", program_name, reason);
+    fprintf (status == 0 ? stdout : stderr, "%s: %s\n",
+	     program_name, reason);
 
-  fprintf (stderr, "\
-Usage: %s [-abcklsxDLS] [--all] [--total] [--count-links] [--summarize]\n\
-       [--bytes] [--kilobytes] [--one-file-system] [--separate-dirs]\n\
-       [--dereference] [--dereference-args] [--help] [--version] [path...]\n",
-	   program_name);
-
-  exit (2);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... [PATH]...\n", program_name);
+      printf ("\
+\n\
+  -a, --all                write counts for all files, not just directories\n\
+  -b, --bytes              print size in bytes\n\
+  -c, --total              produce a grand total\n\
+  -k, --kilobytes          use 1024 blocks, not 512 despite POSIXLY_CORRECT\n\
+  -l, --count-links        count sizes many times if hard linked\n\
+  -s, --summarize          display only a total for each argument\n\
+  -x, --one-file-system    skip directories on different filesystems\n\
+  -D, --dereference-args   dereference PATHs when symbolic link\n\
+  -L, --dereference        dereference all symbolic links\n\
+  -S, --separate-dirs      do not include size of subdirectories\n\
+      --help               display this help and exit\n\
+      --version            output version information and exit\n");
+    }
+  exit (status);
 }
 
 void
@@ -254,21 +283,21 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ((char *) 0);
+	  usage (2, (char *) 0);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage (NULL);
+  if (show_help)
+    usage (0, NULL);
 
   if (opt_all && opt_summarize_only)
-    usage ("cannot both summarize and show all entries");
+    usage (2, "cannot both summarize and show all entries");
 
   /* Initialize the hash structure for inode numbers.  */
   hash_init (INITIAL_HASH_MODULE, INITIAL_ENTRY_TAB_SIZE);
