@@ -52,6 +52,17 @@
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <ctype.h>
@@ -91,6 +102,7 @@ int wait ();
 #define READ_SIZE (32 * 1024)
 
 char *basename ();
+char *stpcpy ();
 char *xmalloc ();
 void error ();
 int make_path ();
@@ -133,10 +145,10 @@ static int strip_files;
 static int dir_arg;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -145,8 +157,8 @@ static struct option const long_options[] =
   {"group", required_argument, NULL, 'g'},
   {"mode", required_argument, NULL, 'm'},
   {"owner", required_argument, NULL, 'o'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -192,24 +204,24 @@ main (argc, argv)
 	  owner_name = optarg;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   /* Check for invalid combinations of arguments. */
   if ((dir_arg && strip_files)
       || (optind == argc)
       || (optind == argc - 1 && !dir_arg))
-    usage ();
+    usage (1);
 
   if (symbolic_mode)
     {
@@ -243,7 +255,7 @@ main (argc, argv)
       else
 	{
 	  if (!isdir (argv[argc - 1]))
-	    usage ();
+	    usage (1);
 	  for (; optind < argc - 1; ++optind)
 	    {
 	      errors |= install_file_in_dir (argv[optind], argv[argc - 1]);
@@ -263,11 +275,17 @@ install_file_in_file (from, to)
      char *from;
      char *to;
 {
-  if (copy_file (from, to))
+  int to_created;
+  int no_need_to_chown;
+
+  if (copy_file (from, to, &to_created))
     return 1;
   if (strip_files)
     strip (to);
-  return change_attributes (to);
+  no_need_to_chown = (to_created
+		      && owner_name == NULL
+		      && group_name == NULL);
+  return change_attributes (to, no_need_to_chown);
 }
 
 /* Copy file FROM into directory TO_DIR, keeping its same name,
@@ -285,7 +303,7 @@ install_file_in_dir (from, to_dir)
 
   from_base = basename (from);
   to = xmalloc ((unsigned) (strlen (to_dir) + strlen (from_base) + 2));
-  sprintf (to, "%s/%s", to_dir, from_base);
+  stpcpy (stpcpy (stpcpy (to, to_dir), "/"), from_base);
   ret = install_file_in_file (from, to);
   free (to);
   return ret;
@@ -295,17 +313,22 @@ install_file_in_dir (from, to_dir)
 static char buffer[READ_SIZE];
 
 /* Copy file FROM onto file TO, creating TO if necessary.
-   Return 0 if the copy is successful, 1 if not. */
+   Return 0 if the copy is successful, 1 if not.  If the copy is
+   successful, set *TO_CREATED to non-zero if TO was created (if it did
+   not exist or did, but was unlinked) and to zero otherwise.  If the
+   copy fails, don't modify *TO_CREATED.  */
 
 static int
-copy_file (from, to)
+copy_file (from, to, to_created)
      char *from;
      char *to;
+     int *to_created;
 {
   int fromfd, tofd;
   int bytes;
   int ret = 0;
   struct stat from_stats, to_stats;
+  int target_created = 1;
 
   if (stat (from, &from_stats))
     {
@@ -331,7 +354,8 @@ copy_file (from, to)
 	  return 1;
 	}
       /* If unlink fails, try to proceed anyway.  */
-      unlink (to);
+      if (unlink (to))
+	target_created = 0;
     }
 
   fromfd = open (from, O_RDONLY, 0);
@@ -373,6 +397,8 @@ copy_file (from, to)
       error (0, errno, "%s", to);
       ret = 1;
     }
+  if (ret == 0)
+    *to_created = target_created;
   return ret;
 
  copy_error:
@@ -382,11 +408,13 @@ copy_file (from, to)
 }
 
 /* Set the attributes of file or directory PATH.
+   If NO_NEED_TO_CHOWN is non-zero, don't call chown.
    Return 0 if successful, 1 if not. */
 
 static int
-change_attributes (path)
+change_attributes (path, no_need_to_chown)
      char *path;
+     int no_need_to_chown;
 {
   int err = 0;
 
@@ -406,7 +434,7 @@ change_attributes (path)
      want to know.  But AFS returns EPERM when you try to change a
      file's group; thus the kludge.  */
 
-  if (chown (path, owner_id, group_id)
+  if (!no_need_to_chown && chown (path, owner_id, group_id)
 #ifdef AFS
       && errno != EPERM
 #endif
@@ -509,15 +537,30 @@ isnumber (str)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-   fprintf (stderr, "\
-Usage: %s [options] [-s] [--strip] source dest\n\
-       %s [options] [-s] [--strip] source... directory\n\
-       %s [options] {-d,--directory} directory...\n\
-Options:\n\
-       [-c] [-g group] [-m mode] [-o owner] [--group=group]\n\
-       [--help] [--version] [--mode=mode] [--owner=owner]\n",
-	    program_name, program_name, program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... SOURCE DEST           (1st format)\n\
+  or:  %s [OPTION]... SOURCE... DIRECTORY   (2nd format)\n\
+  or:  %s [OPTION]... DIRECTORY...          (3nd format)\n\
+",
+	      program_name, program_name, program_name);
+      printf ("\
+\n\
+  -c                  (ignored)\n\
+  -d, --directory     create [leading] directories, mandatory for 3rd format\n\
+  -g, --group GROUP   set group ownership, instead of process' current group\n\
+  -m, --mode MODE     set permission mode (as in chmod), instead of 0755\n\
+  -o, --owner OWNER   set ownership (super-user only)\n\
+  -s, --strip         strip symbol tables, only for 1st and 2nd formats\n\
+      --help          display this help and exit\n\
+      --version       output version information and exit\n");
+    }
+  exit (status);
 }
