@@ -20,6 +20,18 @@
 #ifdef _AIX
  #pragma alloca
 #endif
+
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <getopt.h>
@@ -33,12 +45,33 @@ int link ();			/* Some systems don't declare this anywhere. */
 int symlink ();
 #endif
 
+/* Construct a string NEW_DEST by concatenating DEST, a slash, and
+   basename(SOURCE) in alloca'd memory.  Don't modify DEST or SOURCE.  */
+
+#define PATH_BASENAME_CONCAT(new_dest, dest, source)			\
+    do									\
+      {									\
+	char *source_base;						\
+	char *tmp_source;						\
+									\
+	tmp_source = (char *) alloca (strlen ((source)) + 1);		\
+	strcpy (tmp_source, (source));					\
+	strip_trailing_slashes (tmp_source);				\
+	source_base = basename (tmp_source);				\
+									\
+	(new_dest) = (char *) alloca (strlen ((dest)) + 1		\
+				      + strlen (source_base) + 1);	\
+	stpcpy (stpcpy (stpcpy ((new_dest), (dest)), "/"), source_base);\
+      }									\
+    while (0)
+
 char *basename ();
 enum backup_type get_version ();
 int isdir ();
 int yesno ();
 void error ();
 void strip_trailing_slashes ();
+char *stpcpy ();
 
 static void usage ();
 static int do_link ();
@@ -66,12 +99,12 @@ static int verbose;
 static int hard_dir_link;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
-static struct option const long_options[] = 
+static struct option const long_options[] =
 {
   {"backup", no_argument, NULL, 'b'},
   {"directory", no_argument, &hard_dir_link, 1},
@@ -81,8 +114,8 @@ static struct option const long_options[] =
   {"symbolic", no_argument, &symbolic_link, 1},
   {"verbose", no_argument, &verbose, 1},
   {"version-control", required_argument, NULL, 'V'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -145,22 +178,22 @@ main (argc, argv)
 	  version = optarg;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	  break;
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind == argc)
-    usage ();
+    usage (1);
 
   if (make_backups)
     backup_type = get_version (version);
@@ -174,7 +207,30 @@ main (argc, argv)
     errors = do_link (argv[optind], ".");
   else if (optind == argc - 2)
     {
-      errors = do_link (argv[optind], argv[optind + 1]);
+      struct stat source_stats;
+      char *source;
+      char *dest;
+      char *new_dest;
+
+      source = argv[optind];
+      dest = argv[optind + 1];
+
+      /* When the destination is specified with a trailing slash and the
+	 source exists but is not a directory, convert the user's command
+	 `ln source dest/' to `ln source dest/basename(source)'.  */
+
+      if (dest[strlen (dest) - 1] == '/'
+	  && lstat (source, &source_stats) == 0
+	  && !S_ISDIR (source_stats.st_mode))
+	{
+	  PATH_BASENAME_CONCAT (new_dest, dest, source);
+	}
+      else
+	{
+	  new_dest = dest;
+	}
+
+      errors = do_link (source, new_dest);
     }
   else
     {
@@ -215,17 +271,7 @@ do_link (source, dest)
     {
       /* Target is a directory; build the full filename. */
       char *new_dest;
-      char *source_base;
-      char *tmp_source;
-
-      tmp_source = (char *) alloca (strlen (source) + 1);
-      strcpy (tmp_source, source);
-      strip_trailing_slashes (tmp_source);
-
-      source_base = basename (tmp_source);
-      new_dest = (char *)
-	alloca (strlen (source_base) + 1 + strlen (dest) + 1);
-      sprintf (new_dest, "%s/%s", dest, source_base);
+      PATH_BASENAME_CONCAT (new_dest, dest, source);
       dest = new_dest;
     }
 
@@ -278,7 +324,7 @@ do_link (source, dest)
       error (0, errno, "%s", dest);
       return 1;
     }
-       
+
   if (verbose)
     printf ("%s -> %s\n", source, dest);
 
@@ -304,16 +350,38 @@ do_link (source, dest)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [options] source [dest]\n\
-       %s [options] source... directory\n\
-Options:\n\
-       [-bdfisvF] [-S backup-suffix] [-V {numbered,existing,simple}]\n\
-       [--version-control={numbered,existing,simple}] [--backup] [--directory]\n\
-       [--force] [--interactive] [--symbolic] [--verbose]\n\
-       [--suffix=backup-suffix] [--help] [--version]\n",
-	   program_name, program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... SOURCE [DEST]\n\
+  or:  %s [OPTION]... SOURCE... DIRECTORY\n\
+",
+	      program_name, program_name);
+      printf ("\
+\n\
+  -b, --backup                 make backups for removed files\n\
+  -d, -F, --directory          hard link directories (super-user only)\n\
+  -f, --force                  remove existing destinations\n\
+  -i, --interactive            prompt whether to remove destinations\n\
+  -s, --symbolic               make symbolic links, instead of hard links\n\
+  -v, --verbose                print name of each file before linking\n\
+  -S, --suffix SUFFIX          override the usual backup suffix\n\
+  -V, --version-control WORD   override the usual version control\n\
+      --help                   display this help and exit\n\
+      --version                output version information and exit\n\
+\n\
+The backup suffix is ~, unless set with SIMPLE_BACKUP_SUFFIX.  The\n\
+version control may be set with VERSION_CONTROL, values are:\n\
+\n\
+  t, numbered     make numbered backups\n\
+  nil, existing   numbered if numbered backups exist, simple otherwise\n\
+  never, simple   always make simple backups\n");
+    }
+  exit (status);
 }
