@@ -16,7 +16,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Options:
-   -p, --path		Ensure that the given path(s) exist:
+   -p, --parent		Ensure that the given path(s) exist:
 			Make any missing parent directories for each argument.
 			Parent dirs default to umask modified by `u+wx'.
 			Do not consider an argument directory that already
@@ -27,6 +27,17 @@
 
    David MacKenzie <djm@ai.mit.edu>  */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -42,21 +53,22 @@ static void usage ();
 /* The name this program was run with. */
 char *program_name;
 
-/* If nonzero, ensure that a path exists.  */
+/* If nonzero, ensure that all parents of the specified directory exist.  */
 static int path_mode;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"mode", required_argument, NULL, 'm'},
   {"path", no_argument, &path_mode, 1},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"parents", no_argument, &path_mode, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -87,21 +99,21 @@ main (argc, argv)
 	  symbolic_mode = optarg;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind == argc)
-    usage ();
+    usage (1);
 
   newmode = 0777 & ~umask (0);
   parent_mode = newmode | 0300;	/* u+wx */
@@ -130,11 +142,22 @@ main (argc, argv)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-p] [-m mode] [--path] [--mode=mode] [--help] [--version] dir...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION] DIRECTORY...\n", program_name);
+      printf ("\
+\n\
+  -p, --parents     no error if existing, make parent directories as needed\n\
+  -m, --mode MODE   set permission mode (as in chmod), not 0777 - umask\n\
+      --help        display this help and exit\n\
+      --version     output version information and exit\n");
+    }
+  exit (status);
 }
 
