@@ -27,6 +27,17 @@
 
    David MacKenzie <djm@ai.mit.edu>  */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -42,16 +53,16 @@ static void usage ();
 char *program_name;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"mode", required_argument, NULL, 'm'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -78,18 +89,18 @@ main (argc, argv)
 	  symbolic_mode = optarg;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   newmode = 0666 & ~umask (0);
   if (symbolic_mode)
@@ -103,7 +114,7 @@ main (argc, argv)
     }
 
   if (argc - optind != 2 && argc - optind != 4)
-    usage ();
+    usage (1);
 
   /* Only check the first character, to allow mnemonic usage like
      `mknod /dev/rst0 character 18 0'. */
@@ -115,7 +126,7 @@ main (argc, argv)
       error (4, 0, "block special files not supported");
 #else
       if (argc - optind != 4)
-	usage ();
+	usage (1);
       if (mknod (argv[optind], newmode | S_IFBLK,
 		 makedev (atoi (argv[optind + 2]), atoi (argv[optind + 3]))))
 	error (1, errno, "%s", argv[optind]);
@@ -128,7 +139,7 @@ main (argc, argv)
       error (4, 0, "character special files not supported");
 #else
       if (argc - optind != 4)
-	usage ();
+	usage (1);
       if (mknod (argv[optind], newmode | S_IFCHR,
 		 makedev (atoi (argv[optind + 2]), atoi (argv[optind + 3]))))
 	error (1, errno, "%s", argv[optind]);
@@ -140,27 +151,40 @@ main (argc, argv)
       error (4, 0, "fifo files not supported");
 #else
       if (argc - optind != 2)
-	usage ();
+	usage (1);
       if (mkfifo (argv[optind], newmode))
 	error (1, errno, "%s", argv[optind]);
 #endif
       break;
 
     default:
-      usage ();
+      usage (1);
     }
 
   exit (0);
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [options] path {bcu} major minor\n\
-       %s [options] path p\n\
-Options:\n\
-       [-m mode] [--mode=mode] [--help] [--version]\n",
-	   program_name, program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... PATH TYPE [MAJOR MINOR]\n", program_name);
+      printf ("\
+\n\
+  -m, --mode MODE   set permission mode (as in chmod), not 0666 - umask\n\
+      --help        display this help and exit\n\
+      --version     output version information and exit\n\
+\n\
+MAJOR MINOR are forbidden for TYPE p, mandatory otherwise.  TYPE may be:\n\
+\n\
+  b      create a block (buffered) special file\n\
+  c, u   create a character (unbuffered) special file   \n\
+  p      create a FIFO\n");
+    }
+  exit (status);
 }
