@@ -21,14 +21,14 @@
 
    -i, --interactive	Require confirmation from the user before
 			performing any move that would destroy an
-			existing file. 
+			existing file.
 
    -u, --update		Do not move a nondirectory that has an
 			existing destination with the same or newer
-			modification time.  
+			modification time.
 
    -v, --verbose		List the name of each file as it is moved, and
-			the name it is moved to. 
+			the name it is moved to.
 
    -b, --backup
    -S, --suffix
@@ -40,6 +40,18 @@
 #ifdef _AIX
  #pragma alloca
 #endif
+
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -58,6 +70,7 @@ int yesno ();
 void error ();
 void strip_trailing_slashes ();
 int eaccess_stat ();
+char *stpcpy ();
 
 static int copy_reg ();
 static int do_move ();
@@ -88,10 +101,10 @@ static int stdin_tty;
 static uid_t myeuid;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -102,8 +115,8 @@ static struct option const long_options[] =
   {"update", no_argument, &update, 1},
   {"verbose", no_argument, &verbose, 1},
   {"version-control", required_argument, NULL, 'V'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -157,21 +170,21 @@ main (argc, argv)
 	  version = optarg;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (argc < optind + 2)
-    usage ();
+    usage (1);
 
   if (make_backups)
     backup_type = get_version (version);
@@ -188,6 +201,17 @@ main (argc, argv)
   exit (errors);
 }
 
+/* If PATH is an existing directory, return nonzero, else 0.  */
+
+static int
+is_real_dir (path)
+     char *path;
+{
+  struct stat stats;
+
+  return lstat (path, &stats) == 0 && S_ISDIR (stats.st_mode);
+}
+
 /* Move file SOURCE onto DEST.  Handles the case when DEST is a directory.
    Return 0 if successful, 1 if an error occurred.  */
 
@@ -198,7 +222,8 @@ movefile (source, dest)
 {
   strip_trailing_slashes (source);
 
-  if (isdir (dest))
+  if ((dest[strlen (dest) - 1] == '/' && !is_real_dir (source))
+      || isdir (dest))
     {
       /* Target is a directory; build full target filename. */
       char *base;
@@ -206,7 +231,7 @@ movefile (source, dest)
 
       base = basename (source);
       new_dest = (char *) alloca (strlen (dest) + 1 + strlen (base) + 1);
-      sprintf (new_dest, "%s/%s", dest, base);
+      stpcpy (stpcpy (stpcpy (new_dest, dest), "/"), base);
       return do_move (source, new_dest);
     }
   else
@@ -255,7 +280,8 @@ do_move (source, dest)
 	  && eaccess_stat (&dest_stats, W_OK))
 	{
 	  fprintf (stderr, "%s: replace `%s', overriding mode %04o? ",
-		   program_name, dest, dest_stats.st_mode & 07777);
+		   program_name, dest,
+		   (unsigned int) (dest_stats.st_mode & 07777));
 	  if (!yesno ())
 	    return 0;
 	}
@@ -310,7 +336,7 @@ do_move (source, dest)
 
   if (copy_reg (source, dest))
     goto un_backup;
-  
+
   if (unlink (source))
     {
       error (0, errno, "cannot remove `%s'", source);
@@ -339,14 +365,14 @@ copy_reg (source, dest)
   int ofd;
   char buf[1024 * 8];
   int len;			/* Number of bytes read into `buf'. */
-  
+
   if (!S_ISREG (source_stats.st_mode))
     {
       error (0, 0, "cannot move `%s' across filesystems: Not a regular file",
 	     source);
       return 1;
     }
-  
+
   if (unlink (dest) && errno != ENOENT)
     {
       error (0, errno, "cannot remove `%s'", dest);
@@ -371,7 +397,7 @@ copy_reg (source, dest)
     {
       int wrote = 0;
       char *bp = buf;
-      
+
       do
 	{
 	  wrote = write (ofd, bp, len);
@@ -407,7 +433,7 @@ copy_reg (source, dest)
       error (0, errno, "%s", dest);
       return 1;
     }
-  
+
   /* chown turns off set[ug]id bits for non-root,
      so do the chmod last.  */
 
@@ -443,16 +469,37 @@ copy_reg (source, dest)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [options] source dest\n\
-       %s [options] source... directory\n\
-Options:\n\
-       [-bfiuv] [-S backup-suffix] [-V {numbered,existing,simple}]\n\
-       [--backup] [--force] [--interactive] [--update] [--verbose]\n\
-       [--suffix=backup-suffix] [--version-control={numbered,existing,simple}]\n\
-       [--help] [--version]\n",
-	   program_name, program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... SOURCE DEST\n\
+  or:  %s [OPTION]... SOURCE... DIRECTORY\n\
+",
+	      program_name, program_name);
+      printf ("\
+\n\
+  -b, --backup                 make backup before removal\n\
+  -f, --force                  remove existing destinations, never prompt\n\
+  -i, --interactive            prompt before overwrite\n\
+  -u, --update                 move only older or brand new files\n\
+  -v, --verbose                explain what is being done\n\
+  -S, --suffix SUFFIX          override the usual backup suffix\n\
+  -V, --version-control WORD   override the usual version control\n\
+      --help                   display this help and exit\n\
+      --version                output version information and exit\n\
+\n\
+The backup suffix is ~, unless set with SIMPLE_BACKUP_SUFFIX.  The\n\
+version control may be set with VERSION_CONTROL, values are:\n\
+\n\
+  t, numbered     make numbered backups\n\
+  nil, existing   numbered if numbered backups exist, simple otherwise\n\
+  never, simple   always make simple backups  \n");
+    }
+  exit (status);
 }
