@@ -17,6 +17,17 @@
 
 /* Written by Paul Rubin, David MacKenzie, and Richard Stallman. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -78,10 +89,10 @@ static int unlink_dirs;
 static int stdin_tty;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_opts[] =
 {
@@ -90,8 +101,8 @@ static struct option const long_opts[] =
   {"interactive", no_argument, NULL, 'i'},
   {"recursive", no_argument, &recursive, 1},
   {"verbose", no_argument, &verbose, 1},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -134,25 +145,25 @@ main (argc, argv)
 	  verbose = 1;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind == argc)
     {
       if (ignore_missing_files)
 	exit (0);
       else
-	usage ();
+	usage (1);
     }
 
   stdin_tty = isatty (0);
@@ -227,7 +238,7 @@ remove_file (statp)
 	       program_name,
 	       S_ISDIR (statp->st_mode) ? "directory " : "",
 	       pathname,
-	       statp->st_mode & 07777);
+	       (unsigned int) (statp->st_mode & 07777));
       if (!yesno ())
 	return 1;
     }
@@ -273,7 +284,8 @@ remove_dir (statp)
     {
       fprintf (stderr,
 	       "%s: descend directory `%s', overriding mode %04o? ",
-	       program_name, pathname, statp->st_mode & 07777);
+	       program_name, pathname,
+	       (unsigned int) (statp->st_mode & 07777));
       if (!yesno ())
 	return 1;
     }
@@ -346,7 +358,8 @@ clear_directory (statp)
   int pathname_length;		/* Length of `pathname'. */
   ino_t *inode_space;		/* Copy of directory's inodes. */
   ino_t *inodep;		/* Current entry in `inode_space'. */
-  unsigned inode_size;		/* Bytes allocated for `inode_space'. */
+  unsigned n_inodes_allocated;	/* There is space for this many inodes
+					  in `inode_space'. */
   int err = 0;			/* Return status. */
   struct pathstack pathframe;	/* New top of stack. */
   struct pathstack *pp;		/* Temporary. */
@@ -354,8 +367,8 @@ clear_directory (statp)
   name_size = statp->st_size;
   name_space = (char *) xmalloc (name_size);
 
-  inode_size = statp->st_size;
-  inode_space = (ino_t *) xmalloc (inode_size);
+  n_inodes_allocated = (statp->st_size + sizeof (ino_t) - 1) / sizeof (ino_t);
+  inode_space = (ino_t *) xmalloc (n_inodes_allocated * sizeof (ino_t));
 
   do
     {
@@ -398,12 +411,13 @@ clear_directory (statp)
 		}
 	      namep = stpcpy (namep, dp->d_name) + 1;
 
-	      if (inodep == inode_space + inode_size)
+	      if (inodep == inode_space + n_inodes_allocated)
 		{
 		  ino_t *new_inode_space;
 
-		  inode_size += 1024;
-		  new_inode_space = (ino_t *) xrealloc (inode_space, inode_size);
+		  n_inodes_allocated += 1024;
+		  new_inode_space = (ino_t *) xrealloc (inode_space,
+					n_inodes_allocated * sizeof (ino_t));
 		  inodep += new_inode_space - inode_space;
 		  inode_space = new_inode_space;
 		}
@@ -513,11 +527,24 @@ is the same file as\n", program_name, pathname);
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-dfirvR] [--directory] [--force] [--interactive] [--recursive]\n\
-       [--verbose] [--help] [--version] path...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... PATH...\n", program_name);
+      printf ("\
+\n\
+  -d, --directory       unlink directory, even if non-empty (super-user only)\n\
+  -f, --force           ignore nonexistent files, never prompt\n\
+  -i, --interactive     prompt before any removal\n\
+  -v, --verbose         explain what is being done\n\
+  -r, -R, --recursive   remove the contents of directories recursively\n\
+      --help            display this help and exit\n\
+      --version         output version information and exit\n");
+    }
+  exit (status);
 }
