@@ -16,12 +16,23 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Options:
-   -p, --path		Remove any parent dirs that are explicitly mentioned
+   -p, --parent		Remove any parent dirs that are explicitly mentioned
 			in an argument, if they become empty after the
 			argument file is removed.
 
    David MacKenzie <djm@ai.mit.edu>  */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -41,16 +52,17 @@ char *program_name;
 static int empty_paths;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
   {"path", no_argument, &empty_paths, 1},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"parents", no_argument, &empty_paths, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -75,21 +87,21 @@ main (argc, argv)
 	  empty_paths = 1;
 	  break;
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (optind == argc)
-    usage ();
+    usage (1);
 
   for (; optind < argc; ++optind)
     {
@@ -132,9 +144,20 @@ remove_parents (path)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "Usage: %s [-p] [--path] [--help] [--version] dir...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... DIRECTORY...\n", program_name);
+      printf ("\
+\n\
+  -p, --parents   remove explicit parent directories if being emptied\n\
+      --help      display this help and exit\n\
+      --version   output version information and exit\n");
+    }
+  exit (status);
 }
