@@ -24,7 +24,7 @@
    -r, --file=FILE		Use the time and date of reference file FILE.
    -t TIME			Specify time and date in the form
 				`MMDDhhmm[[CC]YY][.ss]'.
-   
+
    If no options are given, -am is the default, using the current time.
    The -r, -t, and -d options are mutually exclusive.  If a file does not
    exist, create it unless -c is given.
@@ -32,6 +32,17 @@
    Written by Paul Rubin, Arnold Robbins, Jim Kingdon, David MacKenzie,
    and Randy Smith. */
 
+#ifdef HAVE_CONFIG_H
+#if defined (CONFIG_BROKETS)
+/* We use <config.h> instead of "config.h" so that a compilation
+   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
+   (which it would do because it found this file in $srcdir).  */
+#include <config.h>
+#else
+#include "config.h"
+#endif
+#endif
+
 #include <stdio.h>
 #include <ctype.h>
 #include <getopt.h>
@@ -93,10 +104,10 @@ static char *ref_file;
 static struct stat ref_stats;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const longopts[] =
 {
@@ -104,8 +115,8 @@ static struct option const longopts[] =
   {"no-create", no_argument, 0, 'c'},
   {"date", required_argument, 0, 'd'},
   {"file", required_argument, 0, 'r'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
@@ -183,24 +194,24 @@ main (argc, argv)
 	  if (i < 0)
 	    {
 	      invalid_arg ("time selector", optarg, i);
-	      usage ();
+	      usage (1);
 	    }
 	  change_times |= time_masks[i];
 	  break;
 
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (change_times == 0)
     change_times = CH_ATIME | CH_MTIME;
@@ -209,7 +220,7 @@ main (argc, argv)
       || (posix_date && flexible_date))
     {
       error (0, 0, "cannot specify times from more than one source");
-      usage ();
+      usage (1);
     }
 
   if (use_ref)
@@ -239,7 +250,7 @@ main (argc, argv)
   if (optind == argc)
     {
       error (0, 0, "file arguments missing");
-      usage ();
+      usage (1);
     }
 
   for (; optind < argc; ++optind)
@@ -293,7 +304,7 @@ touch (file)
 	{
 	  error (0, errno, "%s", file);
 	  return 1;
-	}	
+	}
     }
 
   if (amtime_now)
@@ -326,7 +337,7 @@ touch (file)
 
       status = utime (file, &utb);
     }
-  
+
   if (status)
     {
       error (0, errno, "%s", file);
@@ -365,12 +376,29 @@ utime_now (file, filesize)
 #endif
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-acfm] [-r reference-file] [-t MMDDhhmm[[CC]YY][.ss]]\n\
-       [-d time] [--time={atime,access,use,mtime,modify}] [--date=time]\n\
-       [--file=reference-file] [--no-create] [--help] [--version] file...\n",
-	   program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... FILE...\n", program_name);
+      printf ("\
+\n\
+  -a                     change only the access time\n\
+  -c                     do not create any files\n\
+  -d, --date STRING      parse STRING and use it instead of current time\n\
+  -f                     (ignored)\n\
+  -m                     change only the modification time\n\
+  -r, --file REFERENCE   use this file's times instead of current time\n\
+  -t STAMP               use MMDDhhmm[[CC]YY][.ss] instead of current time\n\
+      --help             display this help and exit\n\
+      --time WORD        access -a, atime -a, mtime -m, modify -m, use -a\n\
+      --version          output version information and exit\n\
+\n\
+STAMP may be used without -t if none of -drt, nor --, are used.\n");
+    }
+  exit (status);
 }
